@@ -36,14 +36,17 @@
 #include "CSSPendingSubstitutionValue.h"
 #include "CSSPropertyParser.h"
 #include "CSSRegisteredCustomProperty.h"
+#include "CSSValuePair.h"
 #include "CSSValuePool.h"
 #include "CustomPropertyRegistry.h"
 #include "Document.h"
+#include "DocumentInlines.h"
 #include "HTMLElement.h"
 #include "PaintWorkletGlobalScope.h"
-#include "RenderStyle.h"
+#include "RenderStyleSetters.h"
 #include "Settings.h"
 #include "StyleBuilderGenerated.h"
+#include "StyleCustomPropertyData.h"
 #include "StyleFontSizeFunctions.h"
 #include "StylePropertyShorthand.h"
 
@@ -81,7 +84,7 @@ inline bool isValidVisitedLinkProperty(CSSPropertyID id)
     return false;
 }
 
-Builder::Builder(RenderStyle& style, BuilderContext&& context, const MatchResult& matchResult, CascadeLevel cascadeLevel, OptionSet<PropertyCascade::PropertyType> includedProperties, const HashSet<AnimatableProperty>* animatedPropertes)
+Builder::Builder(RenderStyle& style, BuilderContext&& context, const MatchResult& matchResult, CascadeLevel cascadeLevel, OptionSet<PropertyCascade::PropertyType> includedProperties, const HashSet<AnimatableCSSProperty>* animatedPropertes)
     : m_cascade(matchResult, cascadeLevel, includedProperties, animatedPropertes)
     , m_state(*this, style, WTFMove(context))
 {
@@ -111,6 +114,8 @@ void Builder::applyHighPriorityProperties()
 {
     applyProperties(firstHighPriorityProperty, lastHighPriorityProperty);
     m_state.updateFont();
+    // This needs to apply before other properties for the `lh` unit, but after updating the font.
+    applyProperties(CSSPropertyLineHeight, CSSPropertyLineHeight);
 }
 
 void Builder::applyNonHighPriorityProperties()
@@ -118,17 +123,18 @@ void Builder::applyNonHighPriorityProperties()
     ASSERT(!m_state.fontDirty());
 
     applyProperties(firstLowPriorityProperty, lastLowPriorityProperty);
-    applyDeferredProperties();
+    applyLogicalGroupProperties();
     // Any referenced custom properties are already resolved. This will resolve the remaining ones.
     applyCustomProperties();
 
     ASSERT(!m_state.fontDirty());
 }
 
-void Builder::applyDeferredProperties()
+void Builder::applyLogicalGroupProperties()
 {
-    for (auto id : m_cascade.deferredPropertyIDs())
-        applyCascadeProperty(m_cascade.deferredProperty(id));
+    // Properties in a logical property group are applied in author specified order which is maintained separately for them.
+    for (auto id : m_cascade.logicalGroupPropertyIDs())
+        applyCascadeProperty(m_cascade.logicalGroupProperty(id));
 }
 
 void Builder::applyProperties(int firstProperty, int lastProperty)
@@ -142,37 +148,58 @@ void Builder::applyProperties(int firstProperty, int lastProperty)
 template<Builder::CustomPropertyCycleTracking trackCycles>
 inline void Builder::applyPropertiesImpl(int firstProperty, int lastProperty)
 {
-    for (int id = firstProperty; id <= lastProperty; ++id) {
-        CSSPropertyID propertyID = static_cast<CSSPropertyID>(id);
-        if (!m_cascade.hasNormalProperty(propertyID))
-            continue;
+    auto applyProperty = [&](size_t index) ALWAYS_INLINE_LAMBDA {
+        CSSPropertyID propertyID = static_cast<CSSPropertyID>(index);
         ASSERT(propertyID != CSSPropertyCustom);
         auto& property = m_cascade.normalProperty(propertyID);
 
-        if (trackCycles == CustomPropertyCycleTracking::Enabled) {
+        if constexpr (trackCycles == CustomPropertyCycleTracking::Enabled) {
             m_state.m_inProgressProperties.set(propertyID);
             applyCascadeProperty(property);
             m_state.m_inProgressProperties.clear(propertyID);
-            continue;
+            return;
         }
 
         // If we don't have any custom properties, then there can't be any cycles.
         applyCascadeProperty(property);
+    };
+
+    if (m_cascade.propertyIsPresent().size() == static_cast<size_t>(lastProperty + 1)) {
+        m_cascade.propertyIsPresent().forEachSetBit(firstProperty, applyProperty);
+        return;
+    }
+
+    for (int id = firstProperty; id <= lastProperty; ++id) {
+        CSSPropertyID propertyID = static_cast<CSSPropertyID>(id);
+        if (!m_cascade.hasNormalProperty(propertyID))
+            continue;
+        applyProperty(id);
     }
 }
 
 void Builder::applyCustomProperties()
 {
-    for (auto& name : m_cascade.customProperties().keys())
-        applyCustomProperty(name);
+    for (auto& [name, value] : m_cascade.customProperties()) {
+        if (m_state.m_appliedCustomProperties.contains(name))
+            continue;
+        applyCustomPropertyImpl(name, value);
+    }
 }
 
 void Builder::applyCustomProperty(const AtomString& name)
 {
-    if (m_state.m_appliedCustomProperties.contains(name) || !m_cascade.customProperties().contains(name))
+    if (m_state.m_appliedCustomProperties.contains(name))
         return;
 
-    auto& property = m_cascade.customProperty(name);
+    auto iterator = m_cascade.customProperties().find(name);
+    if (iterator == m_cascade.customProperties().end())
+        return;
+
+    applyCustomPropertyImpl(name, iterator->value);
+}
+
+void Builder::applyCustomPropertyImpl(const AtomString& name, const PropertyCascade::Property& property)
+{
     if (!property.cssValue[SelectorChecker::MatchDefault])
         return;
 
@@ -183,7 +210,7 @@ void Builder::applyCustomProperty(const AtomString& name)
         auto isNewCycle = m_state.m_inCycleCustomProperties.add(name).isNewEntry;
         if (isNewCycle) {
             // Continue resolving dependencies so we detect cycles for them as well.
-            resolveCustomPropertyValueWithVariableReferences(customPropertyValue.get());
+            resolveCustomPropertyValue(customPropertyValue.get());
         }
         return;
     }
@@ -204,7 +231,7 @@ void Builder::applyCustomProperty(const AtomString& name)
         return CSSCustomPropertyValue::createWithID(name, CSSValueUnset);
     };
 
-    auto resolvedValue = resolveCustomPropertyValueWithVariableReferences(customPropertyValue.get());
+    auto resolvedValue = resolveCustomPropertyValue(customPropertyValue.get());
 
     if (!resolvedValue || m_state.m_inCycleCustomProperties.contains(name))
         resolvedValue = createInvalidOrUnset();
@@ -278,8 +305,14 @@ void Builder::applyProperty(CSSPropertyID id, CSSValue& value, SelectorChecker::
         registeredCustomProperty = m_state.document().customPropertyRegistry().get(name);
     }
 
-    bool isInherit = valueID == CSSValueInherit;
-    bool isInitial = valueID == CSSValueInitial;
+    auto valueType = [&] {
+        if (valueID == CSSValueInherit)
+            return ApplyValueType::Inherit;
+        if (valueID == CSSValueInitial)
+            return ApplyValueType::Initial;
+        return ApplyValueType::Value;
+    }();
+
     bool isUnset = valueID == CSSValueUnset;
     bool isRevert = valueID == CSSValueRevert;
     bool isRevertLayer = valueID == CSSValueRevertLayer;
@@ -296,18 +329,20 @@ void Builder::applyProperty(CSSPropertyID id, CSSValue& value, SelectorChecker::
             // With the rollback cascade built, we need to obtain the property and apply it. If the property is
             // not present, then we behave like "unset." Otherwise we apply the property instead of our own.
             if (customPropertyValue) {
-                if (registeredCustomProperty && registeredCustomProperty->inherits && rollbackCascade->hasCustomProperty(customPropertyValue->name())) {
-                    auto property = rollbackCascade->customProperty(customPropertyValue->name());
-                    applyRollbackCascadeProperty(property, linkMatchMask);
-                    return;
+                if (registeredCustomProperty && registeredCustomProperty->inherits) {
+                    auto iterator = rollbackCascade->customProperties().find(customPropertyValue->name());
+                    if (iterator != rollbackCascade->customProperties().end()) {
+                        applyRollbackCascadeProperty(iterator->value, linkMatchMask);
+                        return;
+                    }
                 }
-            } else if (id < firstDeferredProperty) {
+            } else if (id < firstLogicalGroupProperty) {
                 if (rollbackCascade->hasNormalProperty(id)) {
                     auto& property = rollbackCascade->normalProperty(id);
                     applyRollbackCascadeProperty(property, linkMatchMask);
                     return;
                 }
-            } else if (auto* property = rollbackCascade->lastDeferredPropertyResolvingRelated(id, style.direction(), style.writingMode())) {
+            } else if (auto* property = rollbackCascade->lastPropertyResolvingLogicalPropertyPair(id, style.direction(), style.writingMode())) {
                 applyRollbackCascadeProperty(*property, linkMatchMask);
                 return;
             }
@@ -323,25 +358,19 @@ void Builder::applyProperty(CSSPropertyID id, CSSValue& value, SelectorChecker::
     if (isUnset) {
         // https://drafts.csswg.org/css-cascade-4/#inherit-initial
         // The unset CSS-wide keyword acts as either inherit or initial, depending on whether the property is inherited or not.
-        if (isInheritedProperty())
-            isInherit = true;
-        else
-            isInitial = true;
+        valueType = isInheritedProperty() ? ApplyValueType::Inherit : ApplyValueType::Initial;
     }
 
-    ASSERT(!isInherit || !isInitial); // isInherit -> !isInitial && isInitial -> !isInherit
-
-    if (m_state.applyPropertyToVisitedLinkStyle() && !isValidVisitedLinkProperty(id)) {
+    if (!m_state.applyPropertyToRegularStyle() && !isValidVisitedLinkProperty(id)) {
         // Limit the properties that can be applied to only the ones honored by :visited.
         return;
     }
 
-    if (isInherit && !isInheritedProperty())
+    if (valueType == ApplyValueType::Inherit && !isInheritedProperty())
         style.setHasExplicitlyInheritedProperties();
 
-#if ENABLE(CSS_PAINTING_API)
-    if (is<CSSPaintImageValue>(valueToApply)) {
-        auto& name = downcast<CSSPaintImageValue>(valueToApply.get()).name();
+    if (auto* paintImageValue = dynamicDowncast<CSSPaintImageValue>(valueToApply.get())) {
+        auto& name = paintImageValue->name();
         if (auto* paintWorklet = const_cast<Document&>(m_state.document()).paintWorkletGlobalScopeForName(name)) {
             Locker locker { paintWorklet->paintDefinitionLock() };
             if (auto* registration = paintWorklet->paintDefinitionMap().get(name)) {
@@ -350,9 +379,63 @@ void Builder::applyProperty(CSSPropertyID id, CSSValue& value, SelectorChecker::
             }
         }
     }
-#endif
 
-    BuilderGenerated::applyProperty(id, m_state, valueToApply.get(), isInitial, isInherit, registeredCustomProperty);
+    if (customPropertyValue) {
+        ASSERT(id == CSSPropertyCustom);
+        applyCustomPropertyValue(*customPropertyValue, valueType, registeredCustomProperty);
+        return;
+    }
+
+    if (UNLIKELY(id == CSSPropertySize && valueType == ApplyValueType::Value)) {
+        applyPageSizeDescriptor(valueToApply.get());
+        return;
+    }
+
+    BuilderGenerated::applyProperty(id, m_state, valueToApply.get(), valueType);
+}
+
+void Builder::applyCustomPropertyValue(const CSSCustomPropertyValue& value, ApplyValueType valueType, const CSSRegisteredCustomProperty* registered)
+{
+    auto applyValue = [&](auto& valueToApply) {
+        ASSERT(valueToApply.isResolved());
+
+        bool isInherited = !registered || registered->inherits;
+        state().style().setCustomPropertyValue(valueToApply, isInherited);
+    };
+
+    auto applyInitial = [&] {
+        if (registered && registered->initialValue) {
+            applyValue(*registered->initialValue);
+            return;
+        }
+        auto invalid = CSSCustomPropertyValue::createWithID(value.name(), CSSValueInvalid);
+        applyValue(invalid.get());
+    };
+
+    auto applyInherit = [&] {
+        auto* parentValue = state().parentStyle().inheritedCustomProperties().get(value.name());
+        if (parentValue && !(registered && !registered->inherits)) {
+            applyValue(const_cast<CSSCustomPropertyValue&>(*parentValue));
+            return;
+        }
+        if (auto* nonInheritedParentValue = state().parentStyle().nonInheritedCustomProperties().get(value.name())) {
+            applyValue(const_cast<CSSCustomPropertyValue&>(*nonInheritedParentValue));
+            return;
+        }
+        applyInitial();
+    };
+
+    switch (valueType) {
+    case ApplyValueType::Initial:
+        applyInitial();
+        break;
+    case ApplyValueType::Inherit:
+        applyInherit();
+        break;
+    case ApplyValueType::Value:
+        applyValue(value);
+        break;
+    };
 }
 
 Ref<CSSValue> Builder::resolveVariableReferences(CSSPropertyID propertyID, CSSValue& value)
@@ -361,32 +444,11 @@ Ref<CSSValue> Builder::resolveVariableReferences(CSSPropertyID propertyID, CSSVa
         return value;
 
     auto variableValue = [&]() -> RefPtr<CSSValue> {
-        if (is<CSSPendingSubstitutionValue>(value)) {
-            auto& substitution = downcast<CSSPendingSubstitutionValue>(value);
-            auto shorthandID = substitution.shorthandPropertyId();
-
-            auto resolvedData = substitution.shorthandValue().resolveVariableReferences(m_state);
-            if (!resolvedData)
-                return nullptr;
-
-            ParsedPropertyVector parsedProperties;
-            if (!CSSPropertyParser::parseValue(shorthandID, false, resolvedData->tokens(), substitution.shorthandValue().context(), parsedProperties, StyleRuleType::Style))
-                return nullptr;
-
-            for (auto& property : parsedProperties) {
-                if (property.id() == propertyID)
-                    return property.value();
-            }
-
-            return nullptr;
-        }
+        if (auto* substitution = dynamicDowncast<CSSPendingSubstitutionValue>(value))
+            return substitution->resolveValue(m_state, propertyID);
 
         auto& variableReferenceValue = downcast<CSSVariableReferenceValue>(value);
-        auto resolvedData = variableReferenceValue.resolveVariableReferences(m_state);
-        if (!resolvedData)
-            return nullptr;
-
-        return CSSPropertyParser::parseSingleValue(propertyID, resolvedData->tokens(), variableReferenceValue.context());
+        return variableReferenceValue.resolveSingleValue(m_state, propertyID);
     }();
 
     // https://drafts.csswg.org/css-variables-2/#invalid-variables
@@ -397,22 +459,83 @@ Ref<CSSValue> Builder::resolveVariableReferences(CSSPropertyID propertyID, CSSVa
     return *variableValue;
 }
 
-RefPtr<CSSCustomPropertyValue> Builder::resolveCustomPropertyValueWithVariableReferences(CSSCustomPropertyValue& value)
+RefPtr<const CSSCustomPropertyValue> Builder::resolveCustomPropertyForContainerQueries(const CSSCustomPropertyValue& value)
 {
-    if (!std::holds_alternative<Ref<CSSVariableReferenceValue>>(value.value()))
-        return &value;
+    if (value.containsCSSWideKeyword()) {
+        auto name = value.name();
+        auto* registered = m_state.document().customPropertyRegistry().get(name);
+        bool isInherited = !registered || registered->inherits;
 
-    auto& variableReferenceValue = std::get<Ref<CSSVariableReferenceValue>>(value.value()).get();
+        auto initial = [&]() -> RefPtr<const CSSCustomPropertyValue> {
+            if (registered)
+                return registered->initialValue;
+            return CSSCustomPropertyValue::createWithID(name, CSSValueInvalid);
+        };
+
+        auto inherit = [&]() -> RefPtr<const CSSCustomPropertyValue> {
+            auto parentValue = isInherited
+                ? m_state.parentStyle().inheritedCustomProperties().get(name)
+                : m_state.parentStyle().nonInheritedCustomProperties().get(name);
+            if (parentValue)
+                return parentValue;
+
+            return initial();
+        };
+
+        auto valueId = std::get<CSSValueID>(value.value());
+        switch (valueId) {
+        case CSSValueInitial:
+            return initial();
+        case CSSValueInherit:
+            return inherit();
+        case CSSValueUnset:
+            return isInherited ? inherit() : initial();
+        case CSSValueRevert:
+        case CSSValueRevertLayer:
+            // https://drafts.csswg.org/css-contain-3/#style-container
+            // "Cascade-dependent keywords, such as revert and revert-layer, are invalid as values in a style feature,
+            // and cause the container style query to be false."
+            return nullptr;
+        default:
+            break;
+        }
+        ASSERT_NOT_REACHED();
+        return nullptr;
+    }
+
+    auto resolvedValue = resolveCustomPropertyValue(const_cast<CSSCustomPropertyValue&>(value));
+    if (!resolvedValue)
+        return CSSCustomPropertyValue::createWithID(value.name(), CSSValueInvalid);
+
+    return resolvedValue;
+}
+
+RefPtr<CSSCustomPropertyValue> Builder::resolveCustomPropertyValue(CSSCustomPropertyValue& value)
+{
+    if (value.containsCSSWideKeyword())
+        return &value;
 
     auto name = value.name();
     auto* registered = m_state.document().customPropertyRegistry().get(name);
-    auto& syntax = registered ? registered->syntax : CSSCustomPropertySyntax::universal();
 
-    auto resolvedData = variableReferenceValue.resolveVariableReferences(m_state);
+    if (value.isResolved() && !registered)
+        return &value;
+
+    auto resolvedData = switchOn(value.value(), [&](const Ref<CSSVariableReferenceValue>& variableReferenceValue) {
+        return variableReferenceValue->resolveVariableReferences(m_state);
+    }, [&](const Ref<CSSVariableData>& data) -> RefPtr<CSSVariableData> {
+        return data.ptr();
+    }, [&](auto&) -> RefPtr<CSSVariableData> {
+        return nullptr;
+    });
+
     if (!resolvedData)
         return nullptr;
 
-    auto dependencies = CSSPropertyParser::collectParsedCustomPropertyValueDependencies(syntax, resolvedData->tokens(), variableReferenceValue.context());
+    if (!registered)
+        return CSSCustomPropertyValue::createSyntaxAll(name, *resolvedData);
+
+    auto dependencies = CSSPropertyParser::collectParsedCustomPropertyValueDependencies(registered->syntax, resolvedData->tokens(), resolvedData->context());
 
     // https://drafts.css-houdini.org/css-properties-values-api/#dependency-cycles
     bool hasCycles = false;
@@ -440,7 +563,159 @@ RefPtr<CSSCustomPropertyValue> Builder::resolveCustomPropertyValueWithVariableRe
     if (isFontDependent)
         m_state.updateFont();
 
-    return CSSPropertyParser::parseTypedCustomPropertyValue(name, syntax, resolvedData->tokens(), m_state, variableReferenceValue.context());
+    return CSSPropertyParser::parseTypedCustomPropertyValue(name, registered->syntax, resolvedData->tokens(), m_state, resolvedData->context());
+}
+
+static bool pageSizeFromName(const CSSPrimitiveValue& pageSizeName, const CSSPrimitiveValue* pageOrientation, Length& width, Length& height)
+{
+    auto mmLength = [](double mm) {
+        return CSSPrimitiveValue::create(mm, CSSUnitType::CSS_MM).get().computeLength<Length>({ });
+    };
+
+    auto inchLength = [](double inch) {
+        return CSSPrimitiveValue::create(inch, CSSUnitType::CSS_IN).get().computeLength<Length>({ });
+    };
+
+    static NeverDestroyed<Length> a5Width(mmLength(148));
+    static NeverDestroyed<Length> a5Height(mmLength(210));
+    static NeverDestroyed<Length> a4Width(mmLength(210));
+    static NeverDestroyed<Length> a4Height(mmLength(297));
+    static NeverDestroyed<Length> a3Width(mmLength(297));
+    static NeverDestroyed<Length> a3Height(mmLength(420));
+    static NeverDestroyed<Length> b5Width(mmLength(176));
+    static NeverDestroyed<Length> b5Height(mmLength(250));
+    static NeverDestroyed<Length> b4Width(mmLength(250));
+    static NeverDestroyed<Length> b4Height(mmLength(353));
+    static NeverDestroyed<Length> jisB5Width(mmLength(182));
+    static NeverDestroyed<Length> jisB5Height(mmLength(257));
+    static NeverDestroyed<Length> jisB4Width(mmLength(257));
+    static NeverDestroyed<Length> jisB4Height(mmLength(364));
+    static NeverDestroyed<Length> letterWidth(inchLength(8.5));
+    static NeverDestroyed<Length> letterHeight(inchLength(11));
+    static NeverDestroyed<Length> legalWidth(inchLength(8.5));
+    static NeverDestroyed<Length> legalHeight(inchLength(14));
+    static NeverDestroyed<Length> ledgerWidth(inchLength(11));
+    static NeverDestroyed<Length> ledgerHeight(inchLength(17));
+
+    switch (pageSizeName.valueID()) {
+    case CSSValueA5:
+        width = a5Width;
+        height = a5Height;
+        break;
+    case CSSValueA4:
+        width = a4Width;
+        height = a4Height;
+        break;
+    case CSSValueA3:
+        width = a3Width;
+        height = a3Height;
+        break;
+    case CSSValueB5:
+        width = b5Width;
+        height = b5Height;
+        break;
+    case CSSValueB4:
+        width = b4Width;
+        height = b4Height;
+        break;
+    case CSSValueJisB5:
+        width = jisB5Width;
+        height = jisB5Height;
+        break;
+    case CSSValueJisB4:
+        width = jisB4Width;
+        height = jisB4Height;
+        break;
+    case CSSValueLetter:
+        width = letterWidth;
+        height = letterHeight;
+        break;
+    case CSSValueLegal:
+        width = legalWidth;
+        height = legalHeight;
+        break;
+    case CSSValueLedger:
+        width = ledgerWidth;
+        height = ledgerHeight;
+        break;
+    default:
+        return false;
+    }
+
+    if (pageOrientation) {
+        switch (pageOrientation->valueID()) {
+        case CSSValueLandscape:
+            std::swap(width, height);
+            break;
+        case CSSValuePortrait:
+            // Nothing to do.
+            break;
+        default:
+            return false;
+        }
+    }
+    return true;
+}
+
+void Builder::applyPageSizeDescriptor(CSSValue& value)
+{
+    m_state.style().resetPageSizeType();
+
+    Length width;
+    Length height;
+    auto pageSizeType = PageSizeType::Auto;
+
+    if (auto* pair = dynamicDowncast<CSSValuePair>(value)) {
+        // <length>{2} | <page-size> <orientation>
+        auto* first = dynamicDowncast<CSSPrimitiveValue>(pair->first());
+        auto* second = dynamicDowncast<CSSPrimitiveValue>(pair->second());
+        if (!first || !second)
+            return;
+        if (first->isLength()) {
+            // <length>{2}
+            if (!second->isLength())
+                return;
+            auto conversionData = m_state.cssToLengthConversionData().copyWithAdjustedZoom(1.0f);
+            width = first->computeLength<Length>(conversionData);
+            height = second->computeLength<Length>(conversionData);
+        } else {
+            // <page-size> <orientation>
+            // The value order is guaranteed. See CSSParser::parseSizeParameter.
+            if (!pageSizeFromName(*first, second, width, height))
+                return;
+        }
+        pageSizeType = PageSizeType::Resolved;
+    } else if (auto* primitiveValue = dynamicDowncast<CSSPrimitiveValue>(value)) {
+        // <length> | auto | <page-size> | [ portrait | landscape]
+        if (primitiveValue->isLength()) {
+            // <length>
+            pageSizeType = PageSizeType::Resolved;
+            width = height = primitiveValue->computeLength<Length>(m_state.cssToLengthConversionData().copyWithAdjustedZoom(1.0f));
+        } else {
+            switch (primitiveValue->valueID()) {
+            case CSSValueInvalid:
+                return;
+            case CSSValueAuto:
+                pageSizeType = PageSizeType::Auto;
+                break;
+            case CSSValuePortrait:
+                pageSizeType = PageSizeType::AutoPortrait;
+                break;
+            case CSSValueLandscape:
+                pageSizeType = PageSizeType::AutoLandscape;
+                break;
+            default:
+                // <page-size>
+                pageSizeType = PageSizeType::Resolved;
+                if (!pageSizeFromName(*primitiveValue, nullptr, width, height))
+                    return;
+            }
+        }
+    } else
+        return;
+
+    m_state.style().setPageSizeType(pageSizeType);
+    m_state.style().setPageSize({ WTFMove(width), WTFMove(height) });
 }
 
 const PropertyCascade* Builder::ensureRollbackCascadeForRevert()
